#!/bin/bash
# Author: Steven Shiau <steven _at_ nchc org tw>
# License: GPL
#
# Note: This code is modified from http://cdprojekte.mattiasschlenker.de/Public/DSL-frominitrd/2.2b-0.0/script/pxedsl.sh
# Ref:
# http://news.mattiasschlenker.de/2006/02/22/pxe-booting-damnsmalllinux/#more-10
#
# SL: Small Linux (Damn Small Linux or Puppy Linux)
#
. /opt/drbl/sbin/drbl-conf-functions

#
check_if_root

# Settings
SL_ISO_URL="http://free.nchc.org.tw/puppylinux"
buffer_ratio_for_iso_to_container="1.15"

#
usage() {
  echo "Load the $SL_FULL_NAME to DRBL environment."
  echo "Usage: $0 PUPPY-LINUX-ISO-FILE"
  echo "Ex: $0 puppy-2.11-seamonkey.iso"
}

#
if [ $# -ne 1 ]; then
  usage
  exit 1
fi
SL_ISO="$1"

if [ ! -f "$SL_ISO" ]; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "$SL_ISO is NOT found!"
  echo "You have to prepare the iso file. For example, you can get it via this command:"
  echo "wget $SL_ISO_URL/$SL_ISO"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi

# get SL_OS name from iso file name
if [ "$(echo $SL_ISO | grep -i "dsl")" ]; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
  echo "This ISO file is Damn Small Linux (DSL)."
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  SL_OS="DSL"
elif [ "$(echo $SL_ISO | grep -i "puppy")" ]; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
  echo "This ISO file is Puppy Linux."
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  SL_OS="Puppy"
else
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "Unknown Small Linux distribution! This script only works with Damn Small Linux or Puppy Linux! Program terminated!!!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi

#
case "$SL_OS" in
  DSL|dsl|DamnSmallLinux)
     # SL_NAME: in one word
     SL_NAME="DSL"
     SL_FULL_NAME="Damn Small Linux"
     # In DSL, boot kernel files are linux24 and minirt24.gz in /boot/isolinux/
     bootfiles_path="/boot/isolinux"
     # in DSL, it's cloop file /cdrom/KNOPPIX/KNOPPIX
     rootfile_path="/KNOPPIX"
     rootfile_path_in_initrd="/cdrom/KNOPPIX"
     # ramdisk_size is for client booting use (kernel parameter).
     ramdisk_size="100000"
     client_RAM_required="192 MB"
     ;;
  Puppy|puppy|PuppyLinux)
     # SL_NAME: in one word
     SL_NAME="PuppyLinux"
     SL_FULL_NAME="Puppy Linux"
     # In PuppyLinux, boot kernel files are vmlinuz and initrd.gz in /
     bootfiles_path="/"
     # in PuppyLinux, it's squashfs, name is like pup_211.sfs in /
     rootfile_path="/"
     rootfile_path_in_initrd="/"
     # ramdisk_size is for client booting use (kernel parameter).
     ramdisk_size="110000"
     client_RAM_required="192 MB"
     ;;
esac

#
ask_and_load_lang_set en

if [ -z "$(file $SL_ISO | grep -i "ISO 9660 CD-ROM filesystem data")" ]; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "$SL_ISO is not an ISO 9660 CD-ROM filesystem data file!"
  echo "Program terminated!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi
# Please adjust the paths for original kernel, initrd if you are not
# booting from the normal CD ROM (e.g. Frugal install on USB)

[ ! -f "$SL_ISO" ] && "$SL_ISO is not found! Program terminated!!!" && exit 1
isomnt="$(mktemp -d /tmp/sl.XXXXXX)"
wd="$(mktemp -d drbl_sl_wd.XXXXXX)"

[ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
echo "This script will create a $SL_FULL_NAME diskless server in DRBL environment. Your DRBL client need at least $client_RAM_required of RAM to run it."
echo -n "Press Enter to continue, or Ctrl-C to quit."
[ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
read

# prepare the size for initrd.
iso_size="$(stat -c "%s" $SL_ISO)"
container_need=$(echo "scale=0; $iso_size * $buffer_ratio_for_iso_to_container / 1024.0" | bc -l)

echo "New NBI initrd size (uncompressed): $container_need MB."
echo "Creating the modified ramdisk will take a while, please be patient."

mount -o loop $SL_ISO $isomnt

# UGLY! Here we assume only one gzip file exists, and it's initrd.
echo "Finding the kernel and initrd from iso..."
for i in $isomnt/$bootfiles_path/*; do
  # sometimes it's "Linux kernel x86", sometimes it's "Linux x86 kernel"
  if [ -n "$(file $i | grep -i "kernel" | grep -i "linux")" ]; then
    kernel="$i"
    break
  fi
done
if [ -n "$kernel" ]; then
  echo "Found the kernel: $kernel"
else
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "The kernel does NOT exist! Program terminated!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi

for i in $isomnt/$bootfiles_path/*; do
  if [ -n "$(file $i | grep -i "gzip compressed data")" ]; then
    oldinitrd="$i"
    break
  fi
done
if [ -n "$oldinitrd" ]; then
  echo "Found the initrd: $oldinitrd"
else
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "The initrd does NOT exist! Program terminated!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi
# we rename the NBI kernel and img, like dsl-linux24 and dsl-minirt24.gz
nbi_kernel="${SL_NAME}-${kernel##*/}"
nbi_initrd="${SL_NAME}-${oldinitrd##*/}"

# Unpack initrd
echo 'Creating container...'
dd if=/dev/zero of=$wd/initrd.img bs=1024 count=$container_need
echo 'Formatting container...'
mkfs.ext2 -q -F -L "${SL_NAME}-initrd" $wd/initrd.img
echo 'Unpacking old initrd...'
gunzip -c "$oldinitrd" > $wd/oldinitrd.img
mkdir $wd/initrd.tmp
mkdir $wd/oldinitrd.tmp
# Mount initrd
mount -o loop $wd/initrd.img $wd/initrd.tmp
mount -o loop $wd/oldinitrd.img $wd/oldinitrd.tmp
#
if [ -e $wd/oldinitrd.tmp/PUPPYVERSION ]; then
  SL_VER="$(cat $wd/oldinitrd.tmp/PUPPYVERSION)"
else
  SL_VER="default"
fi

# TODO, merge this to the beginning setting ?
case "$SL_OS" in
  DSL|dsl|DamnSmallLinux)
     rootfile="$rootfile_path/KNOPPIX"
     # DSL does not have any extra file, the only one is KNOPPIX
     extra_sys_file=""
     append_param_in_pxe_cfg="ramdisk_size=$ramdisk_size init=/etc/init lang=us apm=power-off vga=791 initrd=$nbi_initrd nomce noapic quiet BOOT_IMAGE=knoppix frominitrd"
     ;;
  Puppy|puppy|PuppyLinux)
     rootfile="$rootfile_path/pup_${SL_VER}.sfs"
     # Puppy 2.12, there is a /zdrv_212.sfs
     [ -f "$isomnt/$rootfile_path/zdrv_${SL_VER}.sfs" ] && extra_sys_file="$rootfile_path/zdrv_${SL_VER}.sfs"
     append_param_in_pxe_cfg="ramdisk_size=$ramdisk_size root=/dev/ram0 initrd=$nbi_initrd loglevel=3 frominitrd"
     ;;
esac

echo 'Copying content of the old initrd...'
( cp -a $wd/oldinitrd.tmp/* $wd/initrd.tmp )
mkdir -p $wd/initrd.tmp/$rootfile_path_in_initrd
echo 'Copying root image...'
cp -f $isomnt/$rootfile $wd/initrd.tmp/$rootfile_path_in_initrd/
if [ -n "$extra_sys_file" ]; then
  cp -f $isomnt/$extra_sys_file $wd/initrd.tmp/$rootfile_path_in_initrd/
fi
echo "Copying modified linuxrc from $DRBL_SCRIPT_PATH/setup/files/$SL_NAME/$SL_VER/..."
cp -f $DRBL_SCRIPT_PATH/setup/files/$SL_NAME/$SL_VER/linuxrc.$SL_NAME-$SL_VER.drbl $wd/initrd.tmp/linuxrc

# Umount initrd
umount $wd/initrd.tmp
umount $wd/oldinitrd.tmp

# Pack the initrd again:
echo 'Packing initrd...'
gzip -c $wd/initrd.img > $pxecfg_pd/$nbi_initrd

# Remove the initrd
echo 'Removing temporary copy of initrd...'
rm $wd/initrd.img
# Remove the old original initrd
echo 'Removing temporary copy of original initrd...'
rm $wd/oldinitrd.img

# Copy the linux kernel
cp -f "$kernel" $pxecfg_pd/$nbi_kernel

umount $isomnt

# append the config file in pxelinux dir.
if [ -z "$(grep -E "^[[:space:]]*label[[:space:]]+$SL_NAME[[:space:]]*$" $PXELINUX_DIR/default)" ]; then
  echo "Append the $SL_FULL_NAME config in $PXELINUX_DIR/default..."
  cat <<-SL_PXE >> $PXELINUX_DIR/default 

label $SL_NAME
  # MENU DEFAULT
  # MENU HIDE
  MENU LABEL $SL_FULL_NAME (Ramdisk)
  # MENU PASSWD
  KERNEL $nbi_kernel
  APPEND $append_param_in_pxe_cfg
SL_PXE
fi
echo "$msg_delimiter_star_line"
echo "$msg_all_set_you_can_turn_on_clients"
echo "$msg_etherboot_5_4_is_required"

[ -d "$isomnt" -a -n "$isomnt" ] && rm -rf "$isomnt"
[ -d "$wd" -a -n "$wd" ] && rm -rf "$wd"
