#!/bin/bash
# Author: Steven Shiau <steven _at_ nchc org tw>
# License: GPL
#
# PLD Rescue Linux: http://rescuecd.pld-linux.org/
#
. /opt/drbl/sbin/drbl-conf-functions

#
check_if_root

# Settings
SL_ISO_URL="http://rescuecd.pld-linux.org/download/current/x86/"

#
usage() {
  echo "Load the PLD Rescue CD to DRBL environment."
  echo "Usage: $0 PLD-RESCUE-CD-FILE"
  echo "Ex: $0 rescue.iso"
}

#
if [ $# -ne 1 ]; then
  usage
  exit 1
fi
SL_ISO="$1"

if [ ! -f "$SL_ISO" ]; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "$SL_ISO is NOT found!"
  echo "You have to prepare the iso file. For example, you can get it via this command:"
  echo "wget $SL_ISO_URL/$SL_ISO"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi

# get SL_OS name from iso file name
if [ "$(echo $SL_ISO | grep -i "rescue")" ]; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
  echo "This ISO file is PLD Rescue CD."
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  SL_OS="PLD"
else
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "Unknown Small Linux distribution! This script only works with PLD rescue CD! Program terminated!!!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi

#
case "$SL_OS" in
  PLD|pld)
     # SL_NAME: in one word
     SL_NAME="PLD"
     SL_FULL_NAME="PLD Linux Rescue CD"
     # In PLD, boot kernel files are vmlinuz in /boot/isolinux/
     bootfiles_path="/boot/isolinux"
     # in PLDLinux, it's cpio, name is like rescue.cpi in /
     rootfile_path="/"
     rootfile_path_in_initrd="/"
     # ramdisk_size is for client booting use (kernel parameter).
     ramdisk_size="25000"
     client_RAM_required="25 MB"
     ;;
esac

#
ask_and_load_lang_set en

if [ -z "$(file $SL_ISO | grep -i "ISO 9660 CD-ROM filesystem data")" ]; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "$SL_ISO is not an ISO 9660 CD-ROM filesystem data file!"
  echo "Program terminated!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi
# Please adjust the paths for original kernel, initrd if you are not
# booting from the normal CD ROM (e.g. Frugal install on USB)

[ ! -f "$SL_ISO" ] && "$SL_ISO is not found! Program terminated!!!" && exit 1
isomnt="$(mktemp -d /tmp/sl.XXXXXX)"

[ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
echo "This script will create a $SL_FULL_NAME diskless server in DRBL environment. Your DRBL client need at least $client_RAM_required of RAM to run it."
echo -n "Press Enter to continue, or Ctrl-C to quit."
[ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
read

mount -o loop $SL_ISO $isomnt

# UGLY! Here we assume only one gzip file exists, and it's initrd.
echo "Finding the kernel and initrd from iso..."
for i in $isomnt/$bootfiles_path/*; do
  # sometimes it's "Linux kernel x86", sometimes it's "Linux x86 kernel"
  if [ -n "$(file $i | grep -iw "Linux kernel x86 boot executable")" ]; then
    kernel="$i"
    break
  fi
done
if [ -n "$kernel" ]; then
  echo "Found the kernel: $kernel"
else
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "The kernel does NOT exist! Program terminated!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi

for i in $isomnt/$rootfile_path/*; do
  if [ -n "$(file $i | grep -i "ASCII cpio archive")" ]; then
    oldinitrd="$i"
    break
  fi
done
if [ -n "$oldinitrd" ]; then
  echo "Found the initrd: $oldinitrd"
else
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "The initrd does NOT exist! Program terminated!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  exit 1
fi
# we rename the NBI kernel and img, like PLD-vmlinuz and PLD-rescue.cpi
nbi_kernel="${SL_NAME}-${kernel##*/}"
nbi_initrd="${SL_NAME}-${oldinitrd##*/}"

# TODO, merge this to the beginning setting ?
case "$SL_OS" in
  PLD|pld)
     append_param_in_pxe_cfg="ramdisk_size=$ramdisk_size initrd=$nbi_initrd root=/dev/ram0"
     ;;
esac

# Copy the linux kernel
cp -f $kernel $pxecfg_pd/$nbi_kernel
cp -f $oldinitrd $pxecfg_pd/$nbi_initrd
chmod 644 $pxecfg_pd/$nbi_kernel $pxecfg_pd/$nbi_initrd

umount $isomnt

# append the config file in pxelinux dir.
if [ -z "$(grep -E "^[[:space:]]*label[[:space:]]+$SL_NAME[[:space:]]*$" $PXELINUX_DIR/default)" ]; then
  echo "Append the $SL_FULL_NAME config in $PXELINUX_DIR/default..."
  cat <<-SL_PXE >> $PXELINUX_DIR/default 

label $SL_NAME
  # MENU DEFAULT
  # MENU HIDE
  MENU LABEL $SL_FULL_NAME (Ramdisk)
  # MENU PASSWD
  KERNEL $nbi_kernel
  APPEND $append_param_in_pxe_cfg
SL_PXE
fi
echo "$msg_delimiter_star_line"
echo "$msg_all_set_you_can_turn_on_clients"
echo "$msg_etherboot_5_4_is_required"

[ -d "$isomnt" -a -n "$isomnt" ] && rm -rf "$isomnt"
